@@ -46,15 +46,16 @@ struct _ClockData {
 	int prop_showtooltip;
 };
 
-
-static void clock_properties(AppletWidget *, gpointer);
-
-
 typedef struct {
 	GtkWidget *time; /*the time label*/
 	GtkWidget *align; /* Needed for changing the padding */
 } ComputerClock;
 
+
+static void clock_properties (AppletWidget *applet, gpointer data);
+static void clock_about      (AppletWidget *applet, gpointer data);
+
+
 static void
 free_data(GtkWidget * widget, gpointer data)
 {
@@ -327,8 +328,10 @@ make_clock_applet(const gchar * goad_id)
 	GtkWidget *applet;
 
 	applet = applet_widget_new(goad_id);
-	if (!applet)
-		g_error(_("Can't create applet!\n"));
+	if (!applet) {
+		g_warning(_("Can't create applet!\n"));
+		return NULL;
+	}
 
 	cd = g_new(ClockData, 1);
 
@@ -349,7 +352,7 @@ make_clock_applet(const gchar * goad_id)
 	cd->gmt_time = gnome_config_get_int("clock/gmt_time=0");
 	gnome_config_pop_prefix();
 
-	create_clock_widget(cd,applet);
+	create_clock_widget(cd, applet);
 
 	/*we have to bind change_orient before we do applet_widget_add 
 	  since we need to get an initial change_orient signal to set our
@@ -370,13 +373,19 @@ make_clock_applet(const gchar * goad_id)
 			   GTK_SIGNAL_FUNC(applet_save_session),
 			   cd);
 
+	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
+					      "about",
+					      GNOME_STOCK_MENU_ABOUT,
+					      _("About..."),
+					      clock_about,
+					      NULL);
+
 	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
 					      "properties",
 					      GNOME_STOCK_MENU_PROP,
 					      _("Properties..."),
 					      clock_properties,
 					      cd);
-
 	return applet;
 }
 
@@ -645,3 +654,32 @@ clock_properties(AppletWidget * applet, gpointer data)
 
 	gtk_widget_show(cd->props);
 }
+
+static void
+clock_about (AppletWidget *applet, gpointer data)
+{
+	static GtkWidget   *about     = NULL;
+	static const gchar *authors[] =
+	{
+		"Miguel de Icaza <miguel@kernel.org>",
+		"Federico Mena <quartic@gimp.org>",
+		"Stuart Parmenter <pavlov@innerx.net>",
+		NULL
+	};
+
+	if (about != NULL)
+	{
+		gdk_window_show (about->window);
+		gdk_window_raise (about->window);
+		return;
+	}
+	
+	about = gnome_about_new (_("Clock Applet"), "1.0",
+				 _("(c) 1998 the Free Software Foundation"),
+				 authors,
+				 _("The clock applet gives your panel a lightweight and simple display of the date and time"),
+				 NULL);
+	gtk_signal_connect (GTK_OBJECT(about), "destroy",
+			    GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about);
+	gtk_widget_show (about);
+}
