@@ -126,8 +126,13 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 				gnome_execute_async(NULL, 1, &tmp);
 				g_free(tmp);
 			}
-		} else if (strcmp (menu->name, "help") == 0)
-			panel_pbox_help_cb (NULL, 0, "menus.html");
+		} else if (strcmp (menu->name, "help") == 0) {
+			Menu *menu2 = menu->info->data;
+			char *page;
+			page = (menu2->path && strcmp (menu2->path,"."))
+				? "menus.html" : "mainmenu.html";
+			panel_pbox_help_cb (NULL, 0, page);
+		}
 		break;
 	case APPLET_LOCK:
 		if (strcmp (menu->name, "help") == 0)
