@@ -1,5 +1,6 @@
 /* GNOME panel: foobar widget
- * Copyright 1999 Helix Code, Inc.
+ * Copyright 1999,2000 Helix Code, Inc.
+ * Copyright 2000 Eazel, Inc.
  *
  * Author: Jacob Berkman
  *
@@ -17,11 +18,11 @@
 #include "menu.h"
 #include "menu-util.h"
 #include "session.h"
-/*#include "gdk-bufmap.h"*/
 #include "panel-widget.h"
 #include "xstuff.h"
 #include "basep-widget.h"
 #include "panel_config_global.h"
+#include "panel-util.h"
 #include "drawer-widget.h"
 
 #define SMALL_ICON_SIZE 20
@@ -134,7 +135,7 @@ gmc_client (GtkWidget *w, gpointer data)
 	char *v[3] = { "gmc-client" };
 	v[1] = data;
 	if(gnome_execute_async (NULL, 2, v) < 0)
-		gnome_error_dialog(_("Cannot execute the gmc-client program,\n"
+		panel_error_dialog(_("Cannot execute the gmc-client program,\n"
 				     "perhaps gmc is not installed"));
 }
 
@@ -144,9 +145,9 @@ gnomecal_client (GtkWidget *w, gpointer data)
 	char *v[4] = { "gnomecal", "--view" };
 	v[2] = data;
 	if(gnome_execute_async (NULL, 3, v) < 0)
-		gnome_error_dialog(_("Cannot execute the gnome calendar,\n"
+		panel_error_dialog(_("Cannot execute the gnome calendar,\n"
 				     "perhaps it's not installed.\n"
-				     "It is in the gnome-pimp package."));
+				     "It is in the gnome-pim package."));
 }
 
 static GtkWidget *
@@ -317,7 +318,7 @@ append_folder_menu (GtkWidget *menu_bar, const char *label,
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
 
-	gtk_signal_connect (GTK_OBJECT (menu),"show",
+	gtk_signal_connect (GTK_OBJECT (menu), "show",
 			    GTK_SIGNAL_FUNC (submenu_to_display),
 			    NULL);
 }
@@ -382,7 +383,7 @@ append_clock_menu (GtkWidget *menu_bar)
 		append_gnomecal_item (menu, _(cals[i]), cals[i+1]);
 
 	item = gtk_menu_item_new ();
-	label = gtk_label_new (_(""));
+	label = gtk_label_new ("");
 	timeout = gtk_timeout_add (1000, timeout_cb, label);
 	gtk_signal_connect (GTK_OBJECT (label), "destroy",
 			    GTK_SIGNAL_FUNC (timeout_remove),
@@ -411,8 +412,11 @@ foobar_widget_update_winhints (FoobarWidget *foo)
 	gnome_win_hints_set_state (w, WIN_STATE_STICKY |
 				   WIN_STATE_FIXED_POSITION);
 	
-	gnome_win_hints_set_hints (w, GNOME_PANEL_HINTS | WIN_HINTS_DO_NOT_COVER);	
-	gnome_win_hints_set_layer (w, WIN_LAYER_DOCK);
+	gnome_win_hints_set_hints (w, GNOME_PANEL_HINTS |
+				   WIN_HINTS_DO_NOT_COVER);	
+	gnome_win_hints_set_layer (w, global_config.keep_bottom
+				   ? WIN_LAYER_BELOW
+				   : WIN_LAYER_DOCK);
 }
 
 
@@ -472,11 +476,15 @@ foobar_widget_init (FoobarWidget *foo)
 	menu = create_root_menu (TRUE, flags, TRUE, FALSE, FALSE);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
 	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), menuitem);
+	gtk_signal_connect (GTK_OBJECT (menu), "show",
+			    GTK_SIGNAL_FUNC (submenu_to_display),
+			    NULL);
 
+	append_folder_menu  (menu_bar, _("Favorites"), NULL, FALSE, "apps/.");
 	append_folder_menu  (menu_bar, _("Settings"),  NULL, TRUE,
 			     "gnome/apps/Settings/.");
 	append_desktop_menu (menu_bar);
-	append_folder_menu  (menu_bar, _("Favorites"), NULL, FALSE, "apps/.");
+
 	gtk_box_pack_start (GTK_BOX (foo->hbox), menu_bar, FALSE, FALSE, 0);
 	
 	
@@ -512,7 +520,6 @@ foobar_widget_init (FoobarWidget *foo)
 	gtk_widget_show_all (foo->ebox);
 }
 
-#warning This is probably hackish
 static void
 queue_panel_resize (gpointer data, gpointer user_data)
 {
@@ -537,16 +544,24 @@ queue_panel_resizes (GtkWidget *w, gpointer data)
 	g_list_foreach (panel_list, queue_panel_resize, data);
 }
 
+static void
+foobar_destroyed(GtkWidget *w, gpointer data)
+{
+	das_global_foobar = NULL;
+	g_list_foreach (panel_list, queue_panel_resize, data);
+}
+
 GtkWidget *
 foobar_widget_new (void)
 {
 	g_return_val_if_fail (das_global_foobar == NULL, NULL);
 	das_global_foobar =  gtk_type_new (FOOBAR_WIDGET_TYPE);
-	gtk_signal_connect_after (GTK_OBJECT (das_global_foobar), "size-allocate",
+	gtk_signal_connect_after (GTK_OBJECT (das_global_foobar),
+				  "size-allocate",
 				  GTK_SIGNAL_FUNC (queue_panel_resizes), NULL);
 	gtk_signal_connect (GTK_OBJECT (das_global_foobar), "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &das_global_foobar);
+			    GTK_SIGNAL_FUNC (foobar_destroyed),
+			    NULL);
 	return das_global_foobar;
 }
 
