@@ -1,3 +1,9 @@
+/*
+ * GNOME panel x stuff
+ * Copyright 2000 Eazel, Inc.
+ *
+ * Authors: George Lebl
+ */
 #include <config.h>
 #include <gnome.h>
 #include <gdk/gdkx.h>
@@ -34,26 +40,28 @@ static Window compliant_wm_win = None;
 /*list of all panel widgets created*/
 extern GSList *panel_list;
 
-static int
-get_window_id(Window win, char *title, guint32 *wid)
+gboolean
+get_window_id(guint32 window, char *title, guint32 *wid, gboolean depth)
 {
+	Window win = window;
 	Window root_return;
 	Window parent_return;
 	Window *children = NULL;
 	unsigned int nchildren;
 	unsigned int i;
 	char *tit;
-	int ret = FALSE;
-	
+	gboolean ret = FALSE;
+
 	if(XFetchName(GDK_DISPLAY(), win, &tit) && tit) {
-		if(strstr(tit,title)!=NULL) {
+		if(strstr(tit, title)!=NULL) {
 			if(wid) *wid = win;
 			ret = TRUE;
 		}
 		XFree(tit);
 	}
-	
-	if(ret) return TRUE;
+
+	if(!depth || ret)
+		return ret;
 
 	XQueryTree(GDK_DISPLAY(),
 		   win,
@@ -65,7 +73,7 @@ get_window_id(Window win, char *title, guint32 *wid)
 	/*otherwise we got a problem*/
 	if(children) {
 		for(i=0;!ret && i<nchildren;i++)
-			ret=get_window_id(children[i],title,wid);
+			ret=get_window_id(children[i], title, wid, depth);
 		XFree(children);
 	}
 	return ret;
@@ -102,7 +110,6 @@ static void
 try_checking_swallows(guint32 winid)
 {
 	char *tit;
-	gdk_error_trap_push ();
 	if(XFetchName(GDK_DISPLAY(), winid, &tit) &&
 	   tit) {
 		GList *li;
@@ -114,25 +121,28 @@ try_checking_swallows(guint32 winid)
 				gtk_socket_steal(GTK_SOCKET(swallow->socket),
 						 swallow->wid);
 				check_swallows = 
-					g_list_remove(check_swallows,swallow);
+					g_list_remove(check_swallows, swallow);
+				xstuff_reset_need_substructure();
 				break;
 			}
 		}
 		XFree(tit);
 	}
-	gdk_error_trap_pop ();
 }
 
 static void
 go_through_client_list(void)
 {
 	int i;
+	gdk_error_trap_push ();
 	/* just for status dock stuff for now */
 	for(i=0;i<client_list_size;i++) {
 		if(check_swallows)
 			try_checking_swallows(client_list[i]);
 		try_adding_status(client_list[i]);
 	}
+	gdk_flush();
+	gdk_error_trap_pop ();
 }
 
 static int redo_interface_timeout_handle = 0;
@@ -213,16 +223,19 @@ event_filter(GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 	case MapNotify:
 		if (check_swallows) {
 			GList *li;
-			int remove; /* counts the number of NULLs we should remove
-				       from the check_swallows_list */
+			int remove; /* counts the number of NULLs we
+				       should remove from the
+				       check_swallows_list */
 
 			gdk_error_trap_push ();
 
 			remove = 0;
 			for(li = check_swallows; li; li = g_list_next(li)) {
 				Swallow *swallow = li->data;
-				if(get_window_id(xevent->xmap.window,swallow->title,
-						 &(swallow->wid))) {
+				if(get_window_id(xevent->xmap.window,
+						 swallow->title,
+						 &(swallow->wid),
+						 FALSE)) {
 					gtk_socket_steal(GTK_SOCKET(swallow->socket),swallow->wid);
 					li->data = NULL;
 					remove++;
@@ -230,7 +243,10 @@ event_filter(GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 			}
 			while(remove--)
 				check_swallows = g_list_remove(check_swallows,NULL);
+			if(!check_swallows)
+				xstuff_reset_need_substructure();
 
+			gdk_flush();
 			gdk_error_trap_pop ();
 		}
 	}
@@ -241,6 +257,33 @@ event_filter(GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 	return GDK_FILTER_CONTINUE;
 }
 
+void
+xstuff_reset_need_substructure(void)
+{
+	XWindowAttributes attribs = { 0 };
+
+	gdk_error_trap_push ();
+
+	/* select events, we need to trap the kde status thingies anyway */
+	XGetWindowAttributes (GDK_DISPLAY (),
+			      GDK_ROOT_WINDOW (),
+			      &attribs);
+	if (check_swallows) {
+		XSelectInput (GDK_DISPLAY (),
+			      GDK_ROOT_WINDOW (),
+			      attribs.your_event_mask |
+			      SubstructureNotifyMask);
+	} else {
+		XSelectInput (GDK_DISPLAY (),
+			      GDK_ROOT_WINDOW (),
+			      attribs.your_event_mask &
+			      ~SubstructureNotifyMask);
+	}
+	gdk_flush ();
+
+	gdk_error_trap_pop ();
+}
+
 void
 xstuff_init(void)
 {
@@ -268,6 +311,7 @@ xstuff_init(void)
 	XSelectInput (GDK_DISPLAY (),
 		      GDK_ROOT_WINDOW (),
 		      attribs.your_event_mask |
+		      SubstructureNotifyMask |
 		      StructureNotifyMask |
 		      PropertyChangeMask);
 	gdk_flush ();
@@ -289,6 +333,7 @@ xstuff_set_simple_hint(GdkWindow *w, GdkAtom atom, int val)
 	gdk_error_trap_push();
 	XChangeProperty(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(w), atom, atom,
 			32, PropModeReplace, (unsigned char*)&val, 1);
+	gdk_flush();
 	gdk_error_trap_pop();
 }
 
